"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.process = exports.transform = void 0;
const globby = __importStar(require("globby"));
const debug_1 = __importDefault(require("debug"));
const fsExtra = __importStar(require("fs-extra"));
const debug = debug_1.default('vue-codemod:rule');
const globalAddConfig = {
    global: {
        add: {},
        update: {
            vue: '^3.1.1',
            vuex: '^4.0.1',
            'vue-router': '^4.0.8',
            'vue-i18n': '^9.1.6'
        },
        delete: { 'vue-template-compiler': '', '@vue/composition-api': '' }
    },
    dependencies: {
        add: {},
        update: {},
        delete: {}
    },
    peerDependencies: {
        add: {},
        update: {},
        delete: {}
    },
    devDependencies: {
        add: {
            '@vue/compiler-sfc': '^3.1.1'
        },
        update: {
            '@vue/cli-plugin-babel': '^4.5.0',
            '@vue/cli-service': '^4.5.0'
        }
    }
};
/**
 * Upgrade the package.json file
 * @returns Whether the conversion was successful
 */
function transform() {
    debug('Find package.json.');
    const resolvedPaths = globby.sync('package.json');
    if (resolvedPaths.length <= 0) {
        console.warn('package.json is not exists.');
        return false;
    }
    let packageObj = fsExtra.readJsonSync(resolvedPaths[0]);
    if (JSON.stringify(packageObj) == JSON.stringify(process(packageObj))) {
        return false;
    }
    packageObj = process(packageObj);
    fsExtra.writeJsonSync(resolvedPaths[0], packageObj, { spaces: 2 });
    return true;
}
exports.transform = transform;
/**
 * Modify the configuration of dependencies
 * @param packageObj package.json source
 */
function process(packageObj) {
    var _a;
    Object.keys(globalAddConfig)
        .filter(key => {
        return key != 'global';
    })
        .forEach(key => {
        if (packageObj[key] != undefined) {
            debug(`Process ${key}`);
            packageObj[key] = processCore(packageObj[key], globalAddConfig.global);
            packageObj[key] = processCore(packageObj[key], globalAddConfig[key]);
        }
    });
    if ((_a = packageObj === null || packageObj === void 0 ? void 0 : packageObj.dependencies) === null || _a === void 0 ? void 0 : _a.hasOwnProperty('element-ui')) {
        delete packageObj.dependencies['element-ui'];
        packageObj.dependencies['element-plus'] = '^1.0.2-beta.55';
    }
    return packageObj;
}
exports.process = process;
/**
 * process package.json
 * @param packageObj dependencies...
 * @param config key of config
 * @returns package.json
 */
function processCore(packageObj, config) {
    config.add &&
        Object.keys(config.add).forEach(key => {
            packageObj[key] = config.add[key];
        });
    config.update &&
        Object.keys(config.update).forEach(key => {
            if (packageObj[key] != undefined) {
                packageObj[key] = config.update[key];
            }
        });
    config.delete &&
        Object.keys(config.delete).forEach(key => {
            delete packageObj[key];
        });
    return packageObj;
}
