"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logOutput = exports.formatterOutput = exports.getCntFunc = exports.VuePushManualList = exports.pushManualList = exports.cliInstance = void 0;
const table_1 = require("table");
const cli_progress_1 = __importDefault(require("cli-progress"));
exports.cliInstance = new cli_progress_1.default.SingleBar({
    format: 'progress [{bar}] {percentage}% | {process} | {value}/{total}',
    clearOnComplete: false,
    linewrap: true,
    fps: 144
}, cli_progress_1.default.Presets.shades_classic);
function pushManualList(path, node, name, suggest, website) {
    var _a, _b, _c, _d, _e, _f;
    let index = 0;
    const filepath = path.split('.');
    if (filepath[filepath.length - 1] === 'vue') {
        index = global.scriptLine - 1;
    }
    else {
        index = 0;
    }
    let line;
    let column;
    if (node === null || node === void 0 ? void 0 : node.loc) {
        line = (_a = node === null || node === void 0 ? void 0 : node.loc) === null || _a === void 0 ? void 0 : _a.start.line;
        column = (_b = node === null || node === void 0 ? void 0 : node.loc) === null || _b === void 0 ? void 0 : _b.start.column;
    }
    else {
        line = (_d = (_c = node === null || node === void 0 ? void 0 : node.value) === null || _c === void 0 ? void 0 : _c.loc) === null || _d === void 0 ? void 0 : _d.start.line;
        column = (_f = (_e = node === null || node === void 0 ? void 0 : node.value) === null || _e === void 0 ? void 0 : _e.loc) === null || _f === void 0 ? void 0 : _f.start.column;
    }
    index = line + index;
    let position = '[' + index + ',' + column + ']';
    const list = {
        path: path,
        position: position,
        name: name,
        suggest: suggest,
        website: website
    };
    global.manualList.push(list);
}
exports.pushManualList = pushManualList;
function VuePushManualList(path, node, name, suggest, website) {
    var _a, _b;
    let position = '[' + ((_a = node === null || node === void 0 ? void 0 : node.loc) === null || _a === void 0 ? void 0 : _a.start.line) + ',' + ((_b = node === null || node === void 0 ? void 0 : node.loc) === null || _b === void 0 ? void 0 : _b.start.column) + ']';
    const list = {
        path: path,
        position: position,
        name: name,
        suggest: suggest,
        website: website
    };
    global.manualList.push(list);
}
exports.VuePushManualList = VuePushManualList;
function getCntFunc(key, outputObj) {
    if (!outputObj) {
        outputObj = { key: 0 };
    }
    if (!outputObj.hasOwnProperty(key)) {
        outputObj[key] = 0;
    }
    function cntFunc(quantity = 1) {
        outputObj[key] += quantity;
    }
    return cntFunc;
}
exports.getCntFunc = getCntFunc;
function formatterOutput(processFilePath, formatter, logger) {
    // normal output
    const processFilePathList = processFilePath.join('\n');
    const totalChanged = Object.keys(global.outputReport).reduce((sum, key) => sum + global.outputReport[key], 0);
    const totalDetected = totalChanged + global.manualList.length;
    const transRate = totalDetected == totalChanged
        ? 100
        : ((100 * totalChanged) / totalDetected).toFixed(2);
    console.log(`\x1B[0m--------------------------------------------------`);
    console.log(`Processed ${processFilePath.length} files:\n${processFilePathList}\n`);
    if (global.manualList.length) {
        console.log(`The list that you need to migrate your codes manually (total: ${global.manualList.length}): `);
        let index = 1;
        global.manualList.forEach(manual => {
            console.log('index:', index++);
            console.log(manual);
        });
    }
    console.log('\n\n\x1B[31;4m%s\x1B[0m', `${totalDetected} places`, `need to be transformed`);
    console.log('\x1B[32;4m%s\x1B[0m', `${totalChanged} places`, `was transformed`);
    console.log(`The transformation rate is \x1B[32;4m${transRate}%\x1B[0m`);
    Object.keys(outputReport).forEach(item => {
        if (!outputReport[item])
            delete outputReport[item];
    });
    if (formatter === 'detail') {
        console.log('The transformation stats: \n');
        console.log(global.outputReport);
    }
    else {
        let tableStr;
        let tableOutput = [['Rule Names', 'Count']];
        for (let i in global.outputReport) {
            tableOutput.push([i, global.outputReport[i]]);
        }
        tableStr = table_1.table(tableOutput, {
            drawHorizontalLine: (lineIndex, rowCount) => {
                return lineIndex === 0 || lineIndex === 1 || lineIndex === rowCount;
            },
            columns: [{ alignment: 'left' }, { alignment: 'center' }]
        });
        console.log('The transformation stats: ');
        console.log(tableStr);
        if (formatter === 'log') {
            logOutput(processFilePathList, processFilePath, totalDetected, totalChanged, transRate, tableStr, logger);
        }
    }
}
exports.formatterOutput = formatterOutput;
function logOutput(processFilePathList, processFilePath, totalDetected, totalChanged, transRate, tableStr, logger) {
    logger.log(`Processed ${processFilePath.length} files:\n${processFilePathList}\n`);
    if (global.manualList.length) {
        logger.log('The list that you need to migrate your codes manually');
        let index = 1;
        global.manualList.forEach(manual => {
            logger.log('index:', index++);
            logger.log(manual);
        });
    }
    logger.log(`\n\n${totalDetected} places`, `need to be transformed`);
    logger.log(`${totalChanged} places`, `was transformed`);
    logger.log(`The transformation rate is ${transRate}%`);
    logger.log('The transformation stats: ');
    logger.log(tableStr);
}
exports.logOutput = logOutput;
