"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyOperation = exports.createTransformAST = void 0;
const parser = __importStar(require("vue-eslint-parser"));
const report_1 = require("./report");
const BOM = '\uFEFF';
function createTransformAST(nodeFilter, fix, ruleName) {
    function findNodes(context) {
        const { file } = context;
        const source = file.source;
        const options = { sourceType: 'module' };
        const ast = parser.parse(source, options);
        let toFixNodes = [];
        let root = ast.templateBody;
        parser.AST.traverseNodes(root, {
            enterNode(node) {
                if (nodeFilter(node)) {
                    toFixNodes.push(node);
                }
            },
            leaveNode(node) { }
        });
        return toFixNodes;
    }
    const transformAST = context => {
        const cntFunc = report_1.getCntFunc(ruleName, global.outputReport);
        let fixOperations = [];
        const { file } = context;
        const source = file.source;
        const toFixNodes = findNodes(context);
        toFixNodes.forEach(node => {
            const operations = fix(node, source);
            if (operations.length) {
                cntFunc();
                fixOperations = fixOperations.concat(operations);
            }
        });
        return fixOperations;
    };
    return transformAST;
}
exports.createTransformAST = createTransformAST;
function astTransformationToVueTransformationModule(transformAST) {
    const transform = (file, options) => {
        const source = file.source;
        const fixOperations = transformAST({ file }, options);
        return applyOperation(source, fixOperations);
    };
    transform.type = 'vueTransformation';
    return transform;
}
exports.default = astTransformationToVueTransformationModule;
/**
 * Modify source files
 * @param sourceCode File's source code
 * @param tempOperations Modify the object
 */
function applyOperation(sourceCode, tempOperations) {
    // clone the array
    const bom = sourceCode.startsWith(BOM) ? BOM : '', text = bom ? sourceCode.slice(1) : sourceCode;
    let lastPos = Number.MIN_VALUE, output = bom;
    let applyOperations = [];
    // The Lodash grouping function is called to group the objects in the array according to range
    let tempOperation = mergeOperations(tempOperations, text);
    if (tempOperation) {
        applyOperations.push(tempOperation);
    }
    for (const operation of applyOperations.sort(compareOperationsByRange)) {
        attemptOperation(operation);
    }
    // all fix were recovered.
    output += text.slice(Math.max(0, lastPos));
    return output;
    /**
     * Try to use the 'operation' from a problem.
     * @param   {Message} problem The message object to apply operations from
     * @returns {boolean}         Whether operation was successfully applied
     */
    function attemptOperation(operation) {
        const start = operation.range[0];
        const end = operation.range[1];
        // Remain it as a problem if it's overlapped or it's a negative range
        if (lastPos >= start || start > end) {
            return false;
        }
        // Remove BOM.
        if ((start < 0 && end >= 0) ||
            (start === 0 && operation.text.startsWith(BOM))) {
            output = '';
        }
        // Make output to this operation.
        output += text.slice(Math.max(0, lastPos), Math.max(0, start));
        output += operation.text;
        lastPos = end;
        return true;
    }
}
exports.applyOperation = applyOperation;
/**
 * Merges the given operations array into one.
 * @param {Operation[]} operations The operations to merge.
 * @param {SourceCode} sourceCode The source code object to get the text between operations.
 * @returns {{text: string, range: number[]}} The merged operations
 */
function mergeOperations(operations, sourceCode) {
    if (operations.length === 0) {
        return null;
    }
    if (operations.length === 1) {
        return operations[0];
    }
    operations.sort(compareOperationsByRange);
    const originalText = sourceCode;
    const start = operations[0].range[0];
    const end = operations[operations.length - 1].range[1];
    let text = '';
    let lastPos = Number.MIN_SAFE_INTEGER;
    for (const operation of operations) {
        if (operation.range[0] < lastPos) {
            continue;
        }
        if (operation.range[0] >= 0) {
            text += originalText.slice(Math.max(0, start, lastPos), operation.range[0]);
        }
        text += operation.text;
        lastPos = operation.range[1];
    }
    text += originalText.slice(Math.max(0, start, lastPos), end);
    return { range: [start, end], text };
}
/**
 * Compares items in a operations array by range.
 * @param {Operation} a The first message.
 * @param {Operation} b The second message.
 * @returns {int} -1 if a comes before b, 1 if a comes after b, 0 if equal.
 * @private
 */
function compareOperationsByRange(a, b) {
    return a.range[0] - b.range[0] || a.range[1] - b.range[1];
}
